@@ -1,7 +1,10 @@
 /*
  * Copyright (C) 2006, 2007, 2008 OpenedHand Ltd.
+ * Copyright (C) 2009 Nokia Corporation, all rights reserved.
  *
  * Author: Jorn Baayen <jorn@openedhand.com>
+ *         Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
+ *                               <zeeshan.ali@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -42,9 +45,6 @@
 #include <sys/socket.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <arpa/inet.h>
-#include <net/if.h>
-#include <ifaddrs.h>
 #include <libsoup/soup-address.h>
 #include <glib/gstdio.h>
 
@@ -59,7 +59,6 @@ G_DEFINE_TYPE (GUPnPContext,
                GSSDP_TYPE_CLIENT);
 
 struct _GUPnPContextPrivate {
-        char        *host_ip;
         guint        port;
 
         guint        subscription_timeout;
@@ -68,22 +67,29 @@ struct _GUPnPContextPrivate {
 
         SoupServer  *server; /* Started on demand */
         char        *server_url;
+
+        GList       *host_path_datas;
 };
 
 enum {
         PROP_0,
-        PROP_HOST_IP,
         PROP_PORT,
         PROP_SERVER,
         PROP_SESSION,
         PROP_SUBSCRIPTION_TIMEOUT
 };
 
-#define LOOPBACK_IP "127.0.0.1"
+typedef struct {
+        char *local_path;
+
+        GRegex *regex;
+} UserAgent;
 
 typedef struct {
         char *local_path;
         char *server_path;
+
+        GList *user_agents;
 } HostPathData;
 
 /*
@@ -102,149 +108,6 @@ make_server_id (void)
                                 VERSION);
 }
 
-/*
- * Get the host IP for the specified interface, or the first up and non-loopback
- * interface if no name is specified.
- */
-static char *
-get_host_ip (const char *name)
-{
-        struct ifaddrs *ifa_list, *ifa;
-        char *ret;
-
-        ret = NULL;
-
-        if (getifaddrs (&ifa_list) != 0) {
-                g_error ("Failed to retrieve list of network interfaces:\n%s\n",
-                         strerror (errno));
-
-                return NULL;
-        }
-
-        for (ifa = ifa_list; ifa != NULL; ifa = ifa->ifa_next) {
-                char ip[INET6_ADDRSTRLEN];
-                const char *p;
-                struct sockaddr_in *s4;
-                struct sockaddr_in6 *s6;
-
-                if (ifa->ifa_addr == NULL)
-                        continue;
-
-                if ((ifa->ifa_flags & IFF_LOOPBACK) ||
-                    !(ifa->ifa_flags & IFF_UP))
-                        continue;
-
-                /* If a name was specified, check it */
-                if (name && strcmp (name, ifa->ifa_name) != 0)
-                        continue;
-                
-                p = NULL;
-
-                switch (ifa->ifa_addr->sa_family) {
-                case AF_INET:
-                        s4 = (struct sockaddr_in *) ifa->ifa_addr;
-                        p = inet_ntop (AF_INET,
-                                       &s4->sin_addr, ip, sizeof (ip));
-                        break;
-                case AF_INET6:
-                        s6 = (struct sockaddr_in6 *) ifa->ifa_addr;
-                        p = inet_ntop (AF_INET6,
-                                       &s6->sin6_addr, ip, sizeof (ip));
-                        break;
-                default:
-                        continue; /* Unknown: ignore */
-                }
-
-                if (p != NULL) {
-                        ret = g_strdup (p);
-                        break;
-                }
-        }
-
-        freeifaddrs (ifa_list);
-
-        if (!ret) {
-                /* Didn't find anything. Let's take the loopback IP. */
-                ret = g_strdup (LOOPBACK_IP);
-        }
-
-        return ret;
-}
-
-/*
- * Get the host IP of the interface used for the default route.  On any error,
- * the first up and non-loopback interface is used.
- */
-static char *
-get_default_host_ip (void)
-{
-        FILE *fp;
-        int ret;
-        char dev[32];
-        unsigned long dest;
-        gboolean found = FALSE;
-        
-#if defined(__FreeBSD__)
-	if ((fp = popen ("netstat -r -f inet -n -W", "r"))) {
-		char buffer[BUFSIZ];
-
-		char destination[32];
-
-		int i;
-		/* Skip the 4 header lines */
-		for (i=0;i<4;i++) {
-			if (!(fgets(buffer, BUFSIZ, fp)))
-				return NULL; /* Can't read */
-
-			if (buffer[strlen(buffer)-1] != '\n') {
-				g_warning("Can't read netstat output!");
-				return NULL;
-			}
-		}
-
-		while (fgets(buffer, BUFSIZ, fp)) {
-			if (buffer[strlen(buffer)-1] != '\n') {
-				g_warning("Can't read netstat output!");
-				return NULL;
-			}
-
-			if (sscanf(buffer, "%s %*s %*s %*d %*d %*d %s %*d", destination, dev) == 2) {
-				if (strcmp("default", destination) == 0) {
-					found = TRUE;
-					break;
-				}
-			}
-		}
-		pclose(fp);
-	}
-#else
-        /* TODO: error checking */
-
-        fp = fopen ("/proc/net/route", "r");
-        
-        /* Skip the header */
-        if (fscanf (fp, "%*[^\n]\n") == EOF) {
-               fclose (fp);
-               return NULL;
-	}
-
-        while ((ret = fscanf (fp,
-                              "%31s %lx %*x %*X %*d %*d %*d %*x %*d %*d %*d",
-                              dev, &dest)) != EOF) {
-                /* If we got a device name and destination, and the destination
-                   is 0, then we have the default route */
-                if (ret == 2 && dest == 0) {
-                        found = TRUE;
-                        break;
-                }
-        }
-        
-        fclose (fp);
-#endif
-
-        return get_host_ip (found ? dev : NULL);
-}
-
 static void
 gupnp_context_init (GUPnPContext *context)
 {
@@ -267,6 +130,7 @@ gupnp_context_constructor (GType                  type,
 {
 	GObject *object;
 	GUPnPContext *context;
+        char *user_agent;
 
 	object = G_OBJECT_CLASS (gupnp_context_parent_class)->constructor
                 (type, n_props, props);
@@ -279,6 +143,14 @@ gupnp_context_constructor (GType                  type,
                  gssdp_client_get_main_context (GSSDP_CLIENT (context)),
                  NULL);
 
+        user_agent = g_strdup_printf ("%s GUPnP/" VERSION " DLNADOC/1.50",
+                                      g_get_application_name ()? : "");
+        g_object_set (context->priv->session,
+                      SOUP_SESSION_USER_AGENT,
+                      user_agent,
+                      NULL);
+        g_free (user_agent);
+
 	if (g_getenv ("GUPNP_DEBUG")) {
 		SoupLogger *logger;
 		logger = soup_logger_new (SOUP_LOGGER_LOG_BODY, -1);
@@ -299,9 +171,6 @@ gupnp_context_set_property (GObject      *object,
         context = GUPNP_CONTEXT (object);
 
         switch (property_id) {
-        case PROP_HOST_IP:
-                context->priv->host_ip = g_value_dup_string (value);
-                break;
         case PROP_PORT:
                 context->priv->port = g_value_get_uint (value);
                 break;
@@ -325,10 +194,6 @@ gupnp_context_get_property (GObject    *object,
         context = GUPNP_CONTEXT (object);
 
         switch (property_id) {
-        case PROP_HOST_IP:
-                g_value_set_string (value,
-                                    gupnp_context_get_host_ip (context));
-                break;
         case PROP_PORT:
                 g_value_set_uint (value,
                                   gupnp_context_get_port (context));
@@ -370,6 +235,14 @@ gupnp_context_dispose (GObject *object)
                 context->priv->server = NULL;
         }
 
+        while (context->priv->host_path_datas) {
+                HostPathData *data;
+
+                data = (HostPathData *) context->priv->host_path_datas->data;
+
+                gupnp_context_unhost_path (context, data->server_path);
+        }
+
         /* Call super */
         object_class = G_OBJECT_CLASS (gupnp_context_parent_class);
         object_class->dispose (object);
@@ -383,8 +256,6 @@ gupnp_context_finalize (GObject *object)
 
         context = GUPNP_CONTEXT (object);
 
-        g_free (context->priv->host_ip);
-
         g_free (context->priv->server_url);
 
         /* Call super */
@@ -407,24 +278,6 @@ gupnp_context_class_init (GUPnPContextClass *klass)
 
         g_type_class_add_private (klass, sizeof (GUPnPContextPrivate));
 
-        /**
-         * GUPnPContext:host-ip
-         *
-         * The local host's IP address. Set to NULL to autodetect.
-         **/
-        g_object_class_install_property
-                (object_class,
-                 PROP_HOST_IP,
-                 g_param_spec_string ("host-ip",
-                                      "Host IP",
-                                      "The local host's IP address",
-                                      NULL,
-                                      G_PARAM_READWRITE |
-                                      G_PARAM_CONSTRUCT_ONLY |
-                                      G_PARAM_STATIC_NAME |
-                                      G_PARAM_STATIC_NICK |
-                                      G_PARAM_STATIC_BLURB));
-
         /**
          * GUPnPContext:port
          *
@@ -545,12 +398,21 @@ gupnp_context_get_server (GUPnPContext *context)
         g_return_val_if_fail (GUPNP_IS_CONTEXT (context), NULL);
 
         if (context->priv->server == NULL) {
+                const char *ip;
+
+                ip =  gssdp_client_get_host_ip (GSSDP_CLIENT (context));
+                SoupAddress *addr = soup_address_new (ip, context->priv->port);
+                soup_address_resolve_sync (addr, NULL);
+
                 context->priv->server = soup_server_new
                         (SOUP_SERVER_PORT,
                          context->priv->port,
                          SOUP_SERVER_ASYNC_CONTEXT,
                          gssdp_client_get_main_context (GSSDP_CLIENT (context)),
+                         SOUP_SERVER_INTERFACE,
+                         addr,
                          NULL);
+                g_object_unref (addr);
 
                 soup_server_add_handler (context->priv->server, NULL,
                                          default_server_handler, context,
@@ -571,15 +433,15 @@ make_server_url (GUPnPContext *context)
         SoupServer *server;
         guint port;
 
-        if (context->priv->host_ip == NULL)
-                context->priv->host_ip = get_default_host_ip ();
-
         /* What port are we running on? */
         server = gupnp_context_get_server (context);
         port = soup_server_get_port (server);
 
         /* Put it all together */
-        return g_strdup_printf ("http://%s:%u", context->priv->host_ip, port);
+        return g_strdup_printf
+                        ("http://%s:%u",
+                         gssdp_client_get_host_ip (GSSDP_CLIENT (context)),
+                         port);
 }
 
 const char *
@@ -594,24 +456,24 @@ _gupnp_context_get_server_url (GUPnPContext *context)
 /**
  * gupnp_context_new
  * @main_context: A #GMainContext, or %NULL to use the default one
- * @host_ip: The local host's IP address, or %NULL to use the IP address
- * of the first non-loopback network interface.
+ * @interface: The network interface to use, or %NULL to auto-detect.
  * @port: Port to run on, or 0 if you don't care what port is used.
  * @error: A location to store a #GError, or %NULL
  *
- * Create a new #GUPnPContext with the specified @main_context, @host_ip and @port.
+ * Create a new #GUPnPContext with the specified @main_context, @interface and
+ * @port.
  *
  * Return value: A new #GUPnPContext object, or %NULL on an error
  **/
 GUPnPContext *
 gupnp_context_new (GMainContext *main_context,
-                   const char   *host_ip,
+                   const char   *interface,
                    guint         port,
                    GError      **error)
 {
         return g_object_new (GUPNP_TYPE_CONTEXT,
                              "main-context", main_context,
-                             "host-ip", host_ip,
+                             "interface", interface,
                              "port", port,
                              "error", error,
                              NULL);
@@ -624,16 +486,15 @@ gupnp_context_new (GMainContext *main_context,
  * Get the IP address we advertise ourselves as using.
  *
  * Return value: The IP address. This string should not be freed.
+ *
+ * Deprecated:0.12.7: The "host-ip" property has moved to the base class
+ * #GSSDPClient so newer applications should use
+ * #gssdp_client_get_host_ip instead.
  **/
 const char *
 gupnp_context_get_host_ip (GUPnPContext *context)
 {
-        g_return_val_if_fail (GUPNP_IS_CONTEXT (context), NULL);
-
-        if (context->priv->host_ip == NULL)
-                context->priv->host_ip = get_default_host_ip ();
-
-        return context->priv->host_ip;
+        return gssdp_client_get_host_ip (GSSDP_CLIENT (context));
 }
 
 /**
@@ -696,18 +557,44 @@ gupnp_context_get_subscription_timeout (GUPnPContext *context)
  * if any to make sure we append the locale suffix in a canonical way. */
 static char *
 construct_local_path (const char   *requested_path,
+                      const char   *user_agent,
                       HostPathData *host_path_data)
 {
         GString *str;
+        char *local_path;
         int len;
 
+        local_path = NULL;
+
+        if (user_agent != NULL) {
+                GList *node;
+
+                for (node = host_path_data->user_agents;
+                     node;
+                     node = node->next) {
+                        UserAgent *agent;
+
+                        agent = node->data;
+
+                        if (g_regex_match (agent->regex,
+                                           user_agent,
+                                           0,
+                                           NULL)) {
+                                local_path = agent->local_path;
+                        }
+                }
+        }
+
+        if (local_path == NULL)
+                local_path = host_path_data->local_path;
+
         if (!requested_path || *requested_path == 0)
-                return g_strdup (host_path_data->local_path);
+                return g_strdup (local_path);
 
         if (*requested_path != '/')
                 return NULL; /* Absolute paths only */
 
-        str = g_string_new (host_path_data->local_path);
+        str = g_string_new (local_path);
 
         /* Skip the length of the path relative to which @requested_path
          * is specified. */
@@ -771,11 +658,14 @@ host_path_handler (SoupServer        *server,
         GList *locales, *orig_locales;
         GMappedFile *mapped_file;
         GError *error;
+        HostPathData *host_path_data;
+        const char *user_agent;
 
         orig_locales = NULL;
         locales      = NULL;
         local_path   = NULL;
         path_to_open = NULL;
+        host_path_data = (HostPathData *) user_data;
 
         if (msg->method != SOUP_METHOD_GET &&
             msg->method != SOUP_METHOD_HEAD) {
@@ -784,8 +674,11 @@ host_path_handler (SoupServer        *server,
                 goto DONE;
         }
 
+        user_agent = soup_message_headers_get_one (msg->request_headers,
+                                                   "User-Agent");
+
         /* Construct base local path */
-        local_path = construct_local_path (path, (HostPathData *) user_data);
+        local_path = construct_local_path (path, user_agent, host_path_data);
         if (!local_path) {
                 soup_message_set_status (msg, SOUP_STATUS_BAD_REQUEST);
 
@@ -881,10 +774,9 @@ host_path_handler (SoupServer        *server,
                         goto DONE;
                 }
 
-                if (have_range && (length < 0                   ||
-                                   offset < 0                   ||
-                                   length > st.st_size - offset ||
-                                   offset >= st.st_size)) {
+                if (have_range && (length > st.st_size - offset ||
+                                   st.st_size < 0 ||
+                                   (off_t) offset >= st.st_size)) {
                         soup_message_set_status
                                 (msg,
                                  SOUP_STATUS_REQUESTED_RANGE_NOT_SATISFIABLE);
@@ -959,13 +851,36 @@ host_path_handler (SoupServer        *server,
         }
 }
 
-HostPathData *
+static UserAgent *
+user_agent_new (const char *local_path,
+                GRegex     *regex)
+{
+        UserAgent *agent;
+
+        agent = g_slice_new0 (UserAgent);
+
+        agent->local_path = g_strdup (local_path);
+        agent->regex = g_regex_ref (regex);
+
+        return agent;
+}
+
+static void
+user_agent_free (UserAgent *agent)
+{
+        g_free (agent->local_path);
+        g_regex_unref (agent->regex);
+
+        g_slice_free (UserAgent, agent);
+}
+
+static HostPathData *
 host_path_data_new (const char *local_path,
                     const char *server_path)
 {
         HostPathData *path_data;
 
-        path_data = g_slice_new (HostPathData);
+        path_data = g_slice_new0 (HostPathData);
 
         path_data->local_path  = g_strdup (local_path);
         path_data->server_path = g_strdup (server_path);
@@ -973,11 +888,22 @@ host_path_data_new (const char *local_path,
         return path_data;
 }
 
-void
+static void
 host_path_data_free (HostPathData *path_data)
 {
         g_free (path_data->local_path);
         g_free (path_data->server_path);
+        while (path_data->user_agents) {
+                UserAgent *agent;
+
+                agent = path_data->user_agents->data;
+
+                user_agent_free (agent);
+
+                path_data->user_agents = g_list_delete_link (
+                                path_data->user_agents,
+                                path_data->user_agents);
+        }
 
         g_slice_free (HostPathData, path_data);
 }
@@ -1006,13 +932,68 @@ gupnp_context_host_path (GUPnPContext *context,
 
         server = gupnp_context_get_server (context);
 
-        path_data = host_path_data_new (local_path, server_path);
+        path_data = host_path_data_new (local_path,
+                                        server_path);
 
         soup_server_add_handler (server,
                                  server_path,
                                  host_path_handler,
                                  path_data,
-                                 (GDestroyNotify) host_path_data_free);
+                                 NULL);
+
+        context->priv->host_path_datas =
+                g_list_append (context->priv->host_path_datas,
+                               path_data);
+}
+
+static unsigned int
+path_compare_func (HostPathData *path_data,
+                   const char   *server_path)
+{
+        return strcmp (path_data->server_path, server_path);
+}
+
+/**
+ * gupnp_context_host_path_for_agent
+ * @context: A #GUPnPContext
+ * @local_path: Path to the local file or folder to be hosted
+ * @server_path: Web server path already being hosted
+ * @user_agent: The user-agent as a #GRegex.
+ *
+ * Use this method to serve different local path to specific user-agent(s). The
+ * path @server_path must already be hosted by @context.
+ *
+ * Return value: %TRUE on success, %FALSE otherwise.
+ **/
+gboolean
+gupnp_context_host_path_for_agent (GUPnPContext *context,
+                                   const char   *local_path,
+                                   const char   *server_path,
+                                   GRegex       *user_agent)
+{
+        GList *node;
+
+        g_return_val_if_fail (GUPNP_IS_CONTEXT (context), FALSE);
+        g_return_val_if_fail (local_path != NULL, FALSE);
+        g_return_val_if_fail (server_path != NULL, FALSE);
+        g_return_val_if_fail (user_agent != NULL, FALSE);
+
+        node = g_list_find_custom (context->priv->host_path_datas,
+                                   server_path,
+                                   (GCompareFunc) path_compare_func);
+        if (node != NULL) {
+                HostPathData *path_data;
+                UserAgent *agent;
+
+                path_data = (HostPathData *) node->data;
+                agent = user_agent_new (local_path, user_agent);
+
+                path_data->user_agents = g_list_append (path_data->user_agents,
+                                                        agent);
+
+                return TRUE;
+        } else
+                return FALSE;
 }
 
 /**
@@ -1027,11 +1008,24 @@ gupnp_context_unhost_path (GUPnPContext *context,
                            const char   *server_path)
 {
         SoupServer *server;
+        HostPathData *path_data;
+        GList *node;
 
         g_return_if_fail (GUPNP_IS_CONTEXT (context));
         g_return_if_fail (server_path != NULL);
 
         server = gupnp_context_get_server (context);
 
+        node = g_list_find_custom (context->priv->host_path_datas,
+                                   server_path,
+                                   (GCompareFunc) path_compare_func);
+        g_return_if_fail (node != NULL);
+
+        path_data = (HostPathData *) node->data;
+        context->priv->host_path_datas = g_list_delete_link (
+                        context->priv->host_path_datas,
+                        node);
+
         soup_server_remove_handler (server, server_path);
+        host_path_data_free (path_data);
 }
