@@ -35,6 +35,7 @@
 #include "gupnp-context-private.h"
 #include "gupnp-error.h"
 #include "gupnp-error-private.h"
+#include "gupnp-xml-doc.h"
 #include "http-headers.h"
 #include "xml-util.h"
 
@@ -51,7 +52,7 @@ struct _GUPnPServiceInfoPrivate {
 
         SoupURI *url_base;
 
-        XmlDocWrapper *doc;
+        GUPnPXMLDoc *doc;
 
         xmlNode *element;
 
@@ -124,10 +125,7 @@ gupnp_service_info_set_property (GObject      *object,
 
                 break;
         case PROP_DOCUMENT:
-                info->priv->doc = g_value_get_object (value);
-                if (info->priv->doc)
-                        g_object_ref_sink (info->priv->doc);
-
+                info->priv->doc = g_value_dup_object (value);
                 break;
         case PROP_ELEMENT:
                 info->priv->element = g_value_get_pointer (value);
@@ -197,6 +195,7 @@ gupnp_service_info_dispose (GObject *object)
                                                      data->message,
                                                      SOUP_STATUS_CANCELLED);
 
+                        g_object_unref (data->message);
                         get_scpd_url_data_free (data);
 
                         info->priv->pending_gets =
@@ -213,6 +212,8 @@ gupnp_service_info_dispose (GObject *object)
                 g_object_unref (info->priv->doc);
                 info->priv->doc = NULL;
         }
+
+        G_OBJECT_CLASS (gupnp_service_info_parent_class)->dispose (object);
 }
 
 static void
@@ -227,6 +228,8 @@ gupnp_service_info_finalize (GObject *object)
         g_free (info->priv->service_type);
 
         soup_uri_free (info->priv->url_base);
+
+        G_OBJECT_CLASS (gupnp_service_info_parent_class)->finalize (object);
 }
 
 static void
@@ -347,7 +350,7 @@ gupnp_service_info_class_init (GUPnPServiceInfoClass *klass)
                                       "Document",
                                       "The XML document related to this "
                                       "service",
-                                      TYPE_XML_DOC_WRAPPER,
+                                      GUPNP_TYPE_XML_DOC,
                                       G_PARAM_WRITABLE |
                                       G_PARAM_CONSTRUCT_ONLY |
                                       G_PARAM_STATIC_NAME |
@@ -541,8 +544,11 @@ gupnp_service_info_get_event_subscription_url (GUPnPServiceInfo *info)
  * description document (SCPD) provided by the service so it can not be created
  * if the service does not provide an SCPD.
  *
- * Return value: A new #GUPnPServiceIntrospection for this service or %NULL.
- * Unref after use.
+ * Warning: You  should use gupnp_service_info_get_introspection_async()
+ * instead, this function re-enter the GMainloop before returning.
+ *
+ * Return value: (transfer full):  A new #GUPnPServiceIntrospection for this
+ * service or %NULL. Unref after use.
  **/
 GUPnPServiceIntrospection *
 gupnp_service_info_get_introspection (GUPnPServiceInfo *info,
@@ -577,8 +583,6 @@ gupnp_service_info_get_introspection (GUPnPServiceInfo *info,
                 return NULL;
         }
 
-        http_request_set_user_agent (msg);
-
         /* Send off the message */
         session = gupnp_context_get_session (info->priv->context);
 
@@ -714,8 +718,6 @@ gupnp_service_info_get_introspection_async
                 return;
         }
 
-        http_request_set_user_agent (data->message);
-
         data->info      = info;
         data->callback  = callback;
         data->user_data = user_data;
